# -*- coding: utf-8; -*-
# Copyright (C) 2007 Gabriel Falcão <root@gabrielfalcao.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License as
# published by the Free Software Foundation; either version 2 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public
# License along with this program; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA 02111-1307, USA.
import re

from django.http import HttpResponse, HttpResponseRedirect
from django.template import RequestContext
from django.shortcuts import render_to_response, get_object_or_404
from django.conf import settings
from django.core.exceptions import ObjectDoesNotExist

def html_unentities(s):
    s = s.decode('utf-8')
    entities = {}
    newstring = u""
    
    entities[u'&nbsp;'] = u' '
    entities[u'&iexcl;'] = u'¡'
    entities[u'&cent;'] = u'¢'
    entities[u'&pound;'] = u'£'
    entities[u'&curren;'] = u'¤'
    entities[u'&yen;'] = u'¥'
    entities[u'&brvbar;'] = u'¦'
    entities[u'&sect;'] = u'§'
    entities[u'&uml;'] = u'¨'
    entities[u'&copy;'] = u'©'
    entities[u'&ordf;'] = u'ª'
    entities[u'&laquo;'] = u'«'
    entities[u'&not;'] = u'¬'
    entities[u'&shy;'] = u'­'
    entities[u'&reg;'] = u'®'
    entities[u'&macr;'] = u'¯'
    entities[u'&deg;'] = u'°'
    entities[u'&plusmn;'] = u'±'
    entities[u'&sup2;'] = u'²'
    entities[u'&sup3;'] = u'³'
    entities[u'&acute;'] = u'´'
    entities[u'&micro;'] = u'µ'
    entities[u'&para;'] = u'¶'
    entities[u'&middot;'] = u'·'
    entities[u'&cedil;'] = u'¸'
    entities[u'&sup1;'] = u'¹'
    entities[u'&ordm;'] = u'º'
    entities[u'&raquo;'] = u'»'
    entities[u'&frac14;'] = u'¼'
    entities[u'&frac12;'] = u'½'
    entities[u'&frac34;'] = u'¾'
    entities[u'&iquest;'] = u'¿'
    entities[u'&Agrave;'] = u'À'
    entities[u'&Aacute;'] = u'Á'
    entities[u'&Acirc;'] = u'Â'
    entities[u'&Atilde;'] = u'Ã'
    entities[u'&Auml;'] = u'Ä'
    entities[u'&Aring;'] = u'Å'
    entities[u'&AElig;'] = u'Æ'
    entities[u'&Ccedil;'] = u'Ç'
    entities[u'&Egrave;'] = u'È'
    entities[u'&Eacute;'] = u'É'
    entities[u'&Ecirc;'] = u'Ê'
    entities[u'&Euml;'] = u'Ë'
    entities[u'&Igrave;'] = u'Ì'
    entities[u'&Iacute;'] = u'Í'
    entities[u'&Icirc;'] = u'Î'
    entities[u'&Iuml;'] = u'Ï'
    entities[u'&ETH;'] = u'Ð'
    entities[u'&Ntilde;'] = u'Ñ'
    entities[u'&Ograve;'] = u'Ò'
    entities[u'&Oacute;'] = u'Ó'
    entities[u'&Ocirc;'] = u'Ô'
    entities[u'&Otilde;'] = u'Õ'
    entities[u'&Ouml;'] = u'Ö'
    entities[u'&times;'] = u'×'
    entities[u'&Oslash;'] = u'Ø'
    entities[u'&Ugrave;'] = u'Ù'
    entities[u'&Uacute;'] = u'Ú'
    entities[u'&Ucirc;'] = u'Û'
    entities[u'&Uuml;'] = u'Ü'
    entities[u'&Yacute;'] = u'Ý'
    entities[u'&THORN;'] = u'Þ'
    entities[u'&szlig;'] = u'ß'
    entities[u'&agrave;'] = u'à'
    entities[u'&aacute;'] = u'á'
    entities[u'&acirc;'] = u'â'
    entities[u'&atilde;'] = u'ã'
    entities[u'&auml;'] = u'ä'
    entities[u'&aring;'] = u'å'
    entities[u'&aelig;'] = u'æ'
    entities['&ccedil;'] = u'ç'
    entities[u'&egrave;'] = u'è'
    entities[u'&eacute;'] = u'é'
    entities[u'&ecirc;'] = u'ê'
    entities[u'&euml;'] = u'ë'
    entities[u'&igrave;'] = u'ì'
    entities[u'&iacute;'] = u'í'
    entities[u'&icirc;'] = u'î'
    entities[u'&iuml;'] = u'ï'
    entities[u'&eth;'] = u'ð'
    entities[u'&ntilde;'] = u'ñ'
    entities[u'&ograve;'] = u'ò'
    entities[u'&oacute;'] = u'ó'
    entities[u'&ocirc;'] = u'ô'
    entities[u'&otilde;'] = u'õ'
    entities[u'&ouml;'] = u'ö'
    entities[u'&divide;'] = u'÷'
    entities[u'&oslash;'] = u'ø'
    entities[u'&ugrave;'] = u'ù'
    entities[u'&uacute;'] = u'ú'
    entities[u'&ucirc;'] = u'û'
    entities[u'&uuml;'] = u'ü'
    entities[u'&yacute;'] = u'ý'
    entities[u'&thorn;'] = u'þ'
    entities[u'&yuml;'] = u'ÿ'
    entities[u'&quot;'] = u'"'
    entities[u"'"] = '&#39;'
    entities[u'&lt;'] = u'<'
    entities[u'&gt;'] = u'>'
    entities[u'&amp;'] = u'&'
    buffer = ""
    mutex = False
    s_ent = re.findall(r'[&].*?[;]', s)
    for rp in s_ent:
        s = re.sub(rp.replace('&','[&]').replace(';','[;]'), entities[rp], s)
    return s

class ScheduleHammer(object):
    def __init__(self, HTML):
        if isinstance(HTML, str):
            self.S_tag = '[<]li[>].*?[<]font.*?[>]'
            self.E_tag = '[<][/]font[>].*?[<][/]li[>]'
            self.items = re.findall(r'%s(?P<content>.*?)%s'%(self.S_tag,
                self.E_tag), HTML)
        else:
            raise TypeError("ScheduleHammer must take only a string as \
                    parameter")

    def __iter__(self):
        for title in self.items:
            if "<br" in title:
                yield html_unentities(title.split('<br')[0])
            else:
                yield html_unentities(title)

    def __getitem__(self, pos):
        if "<br" in self.items[pos]:
            return html_unentities(self.items[pos].split("<br")[0])
        else:
            return html_unentities(self.items[pos])

class MetaDict:
    """
    Esta classe recebe um dicionario como parametro e adiciona cada
    chave do dicionario em seu atributo, contendo o valor de cada item
    """
    def __init__(self, dictionary):
        if isinstance(dictionary, dict):
            for key, value in dictionary.items():
                setattr(self, key, value)
        else:
            raise RuntimeError('MetaDict class only accepts a dictionary\
                                    as parameter')
    def __iter__(self):
        for value in self.__dict__.values():
            yield value

    def __getitem__(self,index):
        try:
            return self.__dict__[index]
        except:
            return False

    def __repr__(self):
        return ", ".join(self.all())

    def all(self):
        return self.__dict__.keys()

    def first(self):
        return self.__dict__[self.__dict__.keys()[0]]

    def last(self):
        return self.__dict__[self.__dict__.keys()[-1]]

    def slice(self,index):
        try:
            return self.__dict__[self.__dict__.keys()[index]]
        except:
            return False

    def __contains__(self, item):
        return item in self.__dict__.keys() or item in self.__dict__.values()

    def as_dict(self):
        return self.__dict__

    def __len__(self):
        return len(self.__dict__.keys())

class empty:

    def __default__(self):
        return []

    def __getattr__(self,*a):
        return self.__default__

    def __repr__(self):
        return self.__default__
    
    def __nonzero__(self):
        return 1

def get_object_or_list(obj,**kw):
    try:
        return obj(**kw)
    except ObjectDoesNotExist:
        return empty()

def get_object_or_none(obj,**kw):
    try:
        return obj.objects.get(**kw)
    except ObjectDoesNotExist:
        return None
    except AssertionError:
        #hammer! hammer! hammer!
        return empty()#obj.objects.filter(**kw)[0]

def limpa_conversas(visitante, corretor):
    conversas_vc = []
    if isinstance(visitante, Visitante):
        conversas_vc.append(get_object_or_none(Conversa,visitante__id=\
            visitante.id))
    if isinstance(corretor, Corretor):
        conversas_vc.append(get_object_or_none(Conversa,corretor__id=\
            corretor.id))
    [c.delete() for c in conversas_vc if c]

